<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace SGalinski\SgFormsPdf\Event\Listener;

use SGalinski\SgFormsPdf\Finisher\PdfFinisher;
use SGalinski\SgMail\Events\BeforeSendingFormsMailEvent;
use TYPO3\CMS\Core\Core\Environment;
use TYPO3\CMS\Core\Resource\Exception\ExistingTargetFolderException;
use TYPO3\CMS\Core\Resource\Exception\InsufficientFolderAccessPermissionsException;
use TYPO3\CMS\Core\Resource\Exception\InsufficientFolderWritePermissionsException;
use TYPO3\CMS\Core\Resource\ResourceFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Event listener, to attach the generated pdf's to the user/admin mail, depending on the settings
 */
class BeforeSendingFormsMailEventListener {
	protected const MAIL_TO_USER_FINISHER = 'MailToUserFinisher';

	protected const MAIL_TO_ADMIN_FINISHER = 'MailToAdminFinisher';

	/**
	 * the finishers relevant for this event listener
	 */
	protected const RELEVANT_FINISHERS = [self::MAIL_TO_USER_FINISHER, self::MAIL_TO_ADMIN_FINISHER];

	/**
	 * the path within fileadmin, where the temporary files are stored
	 */
	protected const PDF_TEMP_PATH = 'Temp/PdfAttachments';

	/**
	 * @param BeforeSendingFormsMailEvent $event
	 * @return void
	 * @throws InsufficientFolderAccessPermissionsException
	 * @throws ExistingTargetFolderException
	 * @throws InsufficientFolderWritePermissionsException
	 */
	public function __invoke(BeforeSendingFormsMailEvent $event): void {
		$resourceFactory = GeneralUtility::makeInstance(
			ResourceFactory::class
		);
		$tempPdfPath = $this->getTempPdfPath();
		$defaultStorage = $resourceFactory->getDefaultStorage();
		if (!$defaultStorage) {
			return;
		}

		if (!$defaultStorage->hasFolder($tempPdfPath)) {
			$defaultStorage->createFolder($tempPdfPath);
		}

		if (!$defaultStorage->hasFolder($tempPdfPath)) {
			return;
		}

		$tempPdfFolder = $defaultStorage->getFolder($tempPdfPath);

		if ($tempPdfFolder === NULL) {
			return;
		}
		$mailTemplateService = $event->getMailTemplateService();
		$finisherContext = $event->getFinisherContext();
		$currentFinisher = $finisherContext->getFormRuntime()->getCurrentFinisher();

		if (!$currentFinisher) {
			return;
		}

		$currentFinisherIdentifier = $currentFinisher->getFinisherIdentifier();

		if (!in_array($currentFinisherIdentifier, self::RELEVANT_FINISHERS, TRUE)) {
			return;
		}

		// early return if the Pdf object does not exist
		if (!$finisherContext->getFinisherVariableProvider()->offsetExists(PdfFinisher::FINISHER_NAME)) {
			return;
		}

		$isPdfAttachedToAdminMail = $finisherContext->getFinisherVariableProvider()->get(
			PdfFinisher::FINISHER_NAME,
			PdfFinisher::ATTACH_PDF_TO_ADMIN_MAIL,
			FALSE
		);

		// early return, if the current finisher is the admin finisher, but the corresponding option is not enabled
		if ($currentFinisherIdentifier === self::MAIL_TO_ADMIN_FINISHER && !$isPdfAttachedToAdminMail) {
			return;
		}

		$isPdfAttachedToUserMail = $finisherContext->getFinisherVariableProvider()->get(
			PdfFinisher::FINISHER_NAME,
			PdfFinisher::ATTACH_PDF_TO_USER_MAIL,
			FALSE
		);

		// early return, if the current finisher is the user finisher, but the corresponding option is not enabled
		if ($currentFinisherIdentifier === self::MAIL_TO_USER_FINISHER && !$isPdfAttachedToUserMail) {
			return;
		}

		/** @var string $finisherPdfPath */
		$finisherPdfPath = $finisherContext->getFinisherVariableProvider()->get(
			PdfFinisher::FINISHER_NAME,
			PdfFinisher::FINISHER_PDF_PATH
		);

		if (!$finisherPdfPath) {
			return;
		}

		$pdfFile = $tempPdfFolder->getFile(basename($finisherPdfPath));
		// attach the file to the mail service
		$mailTemplateService->addFileResourceAttachment(
			NULL, $pdfFile
		);
	}

	/**
	 * @param bool $absolute
	 * @return string
	 */
	protected function getTempPdfPath(bool $absolute = FALSE): string {
		$relativePath = ($GLOBALS['TYPO3_CONF_VARS']['EXTENSIONS']['sg_mail']['configurationLocation'] ?? 'SgMail') . '/' . self::PDF_TEMP_PATH;

		return $absolute ? Environment::getPublicPath() . '/fileadmin/' . $relativePath : $relativePath;
	}
}
