<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace SGalinski\SgFormsPdf\Finisher;

use SGalinski\SgFormsPdf\Domain\Model\PdfTemplate;
use SGalinski\SgFormsPdf\Domain\Repository\PdfTemplateRepository;
use SGalinski\SgFormsPdf\Service\PdfService;
use TYPO3\CMS\Core\Core\Environment;
use TYPO3\CMS\Core\Resource\FileInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Domain\Model\FileReference;
use TYPO3\CMS\Form\Domain\Finishers\AbstractFinisher;
use TYPO3\CMS\Form\Domain\Model\FormDefinition;
use TYPO3\CMS\Form\Domain\Model\FormElements\FormElementInterface;

/**
 * PDF Finisher class
 */
class PdfFinisher extends AbstractFinisher {
	public const ATTACH_PDF_TO_ADMIN_MAIL = 'isPdfAttachedToAdminMail';
	public const ATTACH_PDF_TO_USER_MAIL = 'isPdfAttachedToUserMail';
	public const OPEN_PDF_IN_NEW_WINDOW = 'openPdfInNewWindow';
	/**
	 * @var string
	 *
	 * the finisher name
	 */
	public const FINISHER_NAME = 'FormToPdfFinisher';
	/**
	 * @var string
	 *
	 * the key for the entry in the FinisherVariableProvider of the generated pdf file path
	 */
	public const FINISHER_PDF_PATH = 'finisherPdf';

	/**
	 * @var PdfTemplateRepository
	 */
	protected PdfTemplateRepository $pdfTemplateRepository;

	/**
	 * @param PdfTemplateRepository $pdfTemplateRepository
	 */
	public function __construct(PdfTemplateRepository $pdfTemplateRepository) {
		$this->pdfTemplateRepository = $pdfTemplateRepository;
	}

	/**
	 * @return void
	 */
	protected function executeInternal(): void {
		$pdfTemplateUid = (int) $this->parseOption('pdfTemplate');
		/** @var PdfTemplate $pdfTemplate */
		$pdfTemplate = $this->pdfTemplateRepository->findByUid($pdfTemplateUid);
		if (!$pdfTemplate || !$pdfTemplate->getUid() || !$pdfTemplate->getFile()) {
			return;
		}

		$pdfFile = $pdfTemplate->getFile()->getOriginalResource();
		if (!$pdfFile) {
			return;
		}

		$absolutePdfFilePath = Environment::getPublicPath() . $pdfFile->getPublicUrl();
		if (!file_exists($absolutePdfFilePath)) {
			return;
		}

		$fieldMappingConfiguration = $this->getMappingConfigurationForForm(
			$this->finisherContext->getFormRuntime()->getFormDefinition()->getIdentifier()
		);

		if ($fieldMappingConfiguration === NULL) {
			return;
		}

		$this->finisherContext->getFinisherVariableProvider()->add(
			self::FINISHER_NAME,
			self::ATTACH_PDF_TO_ADMIN_MAIL,
			(bool) $this->parseOption(self::ATTACH_PDF_TO_ADMIN_MAIL)
		);
		$this->finisherContext->getFinisherVariableProvider()->add(
			self::FINISHER_NAME,
			self::ATTACH_PDF_TO_USER_MAIL,
			(bool) $this->parseOption(self::ATTACH_PDF_TO_USER_MAIL)
		);
		$this->finisherContext->getFinisherVariableProvider()->add(
			self::FINISHER_NAME,
			self::OPEN_PDF_IN_NEW_WINDOW,
			(bool) $this->parseOption(self::OPEN_PDF_IN_NEW_WINDOW)
		);

		$pdfService = GeneralUtility::makeInstance(PdfService::class, $absolutePdfFilePath);
		$pdf = $pdfService->generate($fieldMappingConfiguration, $this->parseForm());
		if ($pdf === NULL) {
			return;
		}

		$pdfTempFile = GeneralUtility::tempnam(PdfService::PDF_TEMP_PREFIX, PdfService::PDF_TEMP_SUFFIX);
		$pdfTempFileAbsolutePath = GeneralUtility::getFileAbsFileName($pdfTempFile);
		$pdf->Output('F', $pdfTempFileAbsolutePath);

		// flattening results in the PDf form fields to be emptied, so we leave it out for now
//		@shell_exec("gs -sDEVICE=pdfwrite -dSubsetFonts=false -dPDFSETTINGS=/default -dNOPAUSE -dBATCH -sOutputFile=" . $pdfFlatTempFile . " " . $pdfTempFile);
//		@shell_exec('pdftocairo -pdf ' . $pdfTempFile . ' ' . $pdfFlatTempFile);
		$baseName = basename($pdfTempFile);
		$fileDestinationPath = $pdfService->getTempPdfPath(TRUE) . '/' . $baseName;

		// move temp file into our temp directory within fileadmin
		if (rename(GeneralUtility::getFileAbsFileName($pdfTempFile), $fileDestinationPath)) {
			// remove old temp file
			GeneralUtility::unlink_tempfile($pdfTempFile);
		}

		// save the absolute path to our generated pdf, in order for other finishers to be able to use it
		$this->finisherContext->getFinisherVariableProvider()->add(
			self::FINISHER_NAME,
			self::FINISHER_PDF_PATH,
			$fileDestinationPath
		);
	}

	/**
	 * @param string $formIdentifier
	 * @return array|null
	 */
	protected function getMappingConfigurationForForm(string $formIdentifier): ?array {
		$configurationManager = GeneralUtility::makeInstance(ConfigurationManagerInterface::class);
		$typoScriptSettings = $configurationManager->getConfiguration(
			ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK,
			'SgFormsPdf'
		);

		return $typoScriptSettings['mappings'][$formIdentifier] ?? NULL;
	}

	/**
	 * @return array
	 */
	protected function parseForm(): array {
		$formValues = [];
		$formDefinition = $this->finisherContext->getFormRuntime()->getFormDefinition();
		if ($formDefinition instanceof FormDefinition) {
			foreach ($this->finisherContext->getFormValues() as $fieldName => $fieldValue) {
				$fieldElement = $formDefinition->getElementByIdentifier($fieldName);
				if ($fieldElement instanceof FormElementInterface && $fieldElement->getType() !== 'Honeypot') {
					if ($fieldValue instanceof FileReference) {
						$originalResource = $fieldValue->getOriginalResource();
						if ($originalResource instanceof \TYPO3\CMS\Core\Resource\FileReference) {
							$originalResource = $originalResource->getOriginalFile();
						}

						if ($originalResource instanceof FileInterface) {
							$fieldValue = $originalResource->getPublicUrl();
						}
					}
					$formValues[$fieldName] = $fieldValue;
				}
			}
		}

		return $formValues;
	}
}
