<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSevent.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace SGalinski\SgFormsPdf\Service;

use SGalinski\SgFormsPdf\Finisher\PdfFinisher;
use TYPO3\CMS\Core\Core\Environment;
use TYPO3\CMS\Core\TypoScript\TypoScriptService;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Service to generate the pdf files from within the PdfFinisher
 *
 * @see PdfFinisher
 */
class PdfService {
	public const PDF_NAME = 'form.pdf';
	public const PDF_TEMP_PREFIX = 'form-generated';
	public const PDF_TEMP_SUFFIX = '.pdf';

	/**
	 * the path within fileadmin, where the temporary files are stored
	 */
	protected const PDF_TEMP_PATH = 'Temp/PdfAttachments';

	/**
	 * @var string
	 */
	protected string $templatePath = '';

	public function __construct(string $templatePath) {
		$this->templatePath = $templatePath;
	}

	/**
	 * @param array $fieldMappings
	 * @param array $values
	 * @return \FPDM|null
	 */
	public function generate(
		array $fieldMappings,
		array $values
	): ?\FPDM {
		if (!$this->getTemplatePath()) {
			return NULL;
		}

		if (!$fieldMappings || !$values) {
			return NULL;
		}

		$formData = [];
		foreach ($fieldMappings as $formFieldName => $pdfFieldNameOrConfig) {
			// parse virtual fields like CURRENT_DATE / CURRENT_TIME
			if ($formFieldName === 'virtualFields') {
				$formData = $this->parseVirtualFields($formData, $pdfFieldNameOrConfig);
			} else if (is_string($pdfFieldNameOrConfig)) {
				$formData[$pdfFieldNameOrConfig] = $values[$formFieldName] ?? '';
			} else if (is_array($pdfFieldNameOrConfig)) {
				if (!array_key_exists('name', $pdfFieldNameOrConfig) || !array_key_exists(
						'type', $pdfFieldNameOrConfig
					)) {
					continue;
				}
				$formData[$pdfFieldNameOrConfig['name']] = $this->parseCustomField(
					$values[$formFieldName], $pdfFieldNameOrConfig['type']
				);
			}
		}

		$pdf = new \FPDM($this->getTemplatePath());
		// fill pdf form with form values
		$pdf->Load($formData, TRUE);
		$pdf->Merge();

		return $pdf;
	}

	/**
	 * @param mixed $formValue
	 * @param string $type
	 * @return string
	 */
	protected function parseCustomField(mixed $formValue, string $type): string {
		if (!$type || !$formValue) {
			return '';
		}

		if ($type === 'DATE') {
			/** @var \DateTime $formValue */
			return $formValue->format('d.m.Y');
		}

		return (string) $formValue;
	}

	/**
	 * @return string
	 */
	public function getTemplatePath(): string {
		return $this->templatePath;
	}

	/**
	 * @param array $formData
	 * @param array $virtualFieldsTyposcriptArray
	 * @return array
	 */
	protected function parseVirtualFields(array $formData, array $virtualFieldsTyposcriptArray): array {
		$typoscriptService = GeneralUtility::makeInstance(TypoScriptService::class);
		$virtualFieldsTyposcriptArray = $typoscriptService->convertTypoScriptArrayToPlainArray(
			$virtualFieldsTyposcriptArray
		);
		foreach ($virtualFieldsTyposcriptArray as $index => $virtualField) {
			if (!array_key_exists('type', $virtualField) || !array_key_exists('name', $virtualField)) {
				continue;
			}

			if ($virtualField['type'] === 'CURRENT_DATE') {
				$formData[$virtualField['name']] = date('d.m.Y');
			} else if ($virtualField['type'] === 'CURRENT_TIME') {
				$formData[$virtualField['name']] = date('H:i');
			}
		}

		return $formData;
	}

	/**
	 * @param bool $absolute
	 * @return string
	 */
	public function getTempPdfPath(bool $absolute = FALSE): string {
		$relativePath = ($GLOBALS['TYPO3_CONF_VARS']['EXTENSIONS']['sg_mail']['configurationLocation'] ?? 'SgMail') . '/' . self::PDF_TEMP_PATH;

		return $absolute ? Environment::getPublicPath() . '/fileadmin/' . $relativePath : $relativePath;
	}
}
